@@ -1,11 +1,9 @@
 import json
-import uuid
-import traceback
 
 from copy import copy
 from .config import get_message_counter_contract
 
-from django.db.models.query import QuerySet, Q
+from django.db.models.query import Q
 from django.conf import settings
 from django.contrib.auth.models import AnonymousUser
 from django.core.serializers.json import DjangoJSONEncoder
@@ -20,7 +18,7 @@ from contract.models import Contract as ContractModel, \
 from calculation.services import run_calculation_rules
 
 from policyholder.models import PolicyHolderInsuree
-from contribution.models import Premium, Payer
+from contribution.models import Premium
 from contribution_plan.models import ContributionPlanBundleDetails, ContributionPlan
 
 from policy.models import Policy
@@ -28,8 +26,14 @@ from payment.models import Payment, PaymentDetail
 from payment.services import update_or_create_payment
 from insuree.models import Insuree
 
+from dateutil.relativedelta import relativedelta
+
 from core.signals import *
 
+import logging
+
+logger = logging.getLogger(__file__)
+
 
 class ContractUpdateError(Exception):
 
@@ -83,11 +87,11 @@ class Contract(object):
                 )["total_amount"]
                 c.amount_notified = total_amount
             historical_record = c.history.all().last()
-            c.json_ext = json.dumps(_save_json_external(
-                user_id=historical_record.user_updated.id,
-                datetime=historical_record.date_updated,
+            c.json_ext = _save_json_external(
+                user_id=str(historical_record.user_updated.id),
+                datetime=str(historical_record.date_updated),
                 message=f"create contract status {historical_record.state}"
-            ), cls=DjangoJSONEncoder)
+            )
             c.save(username=self.user.username)
             dict_representation = model_to_dict(c)
             dict_representation['id'], dict_representation['uuid'] = (str(uuid_string), str(uuid_string))
@@ -103,6 +107,9 @@ class Contract(object):
                 "save": save,
             }
         )
+        if not result_contract_valuation or result_contract_valuation["success"] is False:
+            logger.error("contract valuation failed %s", str(result_contract_valuation))
+            raise Exception("contract valuation failed " + str(result_contract_valuation))
         return result_contract_valuation["data"]
 
     # TODO update contract scenario according to wiki page
@@ -160,11 +167,11 @@ class Contract(object):
         updated_contract.save(username=self.user.username)
         # save the communication
         historical_record = updated_contract.history.all().first()
-        updated_contract.json_ext = json.dumps(_save_json_external(
-            user_id=historical_record.user_updated.id,
-            datetime=historical_record.date_updated,
+        updated_contract.json_ext = _save_json_external(
+            user_id=str(historical_record.user_updated.id),
+            datetime=str(historical_record.date_updated),
             message="update contract status " + str(historical_record.state)
-        ), cls=DjangoJSONEncoder)
+        )
         updated_contract.save(username=self.user.username)
         uuid_string = f"{updated_contract.id}"
         dict_representation = model_to_dict(updated_contract)
@@ -219,7 +226,8 @@ class Contract(object):
     def __gather_policy_holder_insuree(self, contract_details, amendment, contract_date_valid_from=None):
         result = []
         for cd in contract_details:
-            ph_insuree = PolicyHolderInsuree.objects.filter(Q(insuree_id=cd['insuree_id'], last_policy__isnull=False)).first()
+            ph_insuree = PolicyHolderInsuree.objects.filter(
+                Q(insuree_id=cd['insuree_id'], last_policy__isnull=False)).first()
             policy_id = ph_insuree.last_policy.id if ph_insuree else None
             result.append({
                 "id": f"{cd['id']}",
@@ -243,7 +251,7 @@ class Contract(object):
             # variable to check if we have right to approve
             state_right = self.__check_rights_by_status(contract_to_approve.state)
             # check if we can submit
-            if state_right is not "approvable":
+            if state_right != "approvable":
                 raise ContractUpdateError("You cannot approve this contract! The status of contract is not Negotiable!")
             contract_details_list = {}
             contract_details_list["data"] = self.__gather_policy_holder_insuree(
@@ -282,7 +290,7 @@ class Contract(object):
             # variable to check if we have right to approve
             state_right = self.__check_rights_by_status(contract_to_counter.state)
             # check if we can submit
-            if state_right is not "approvable":
+            if state_right != "approvable":
                 raise ContractUpdateError("You cannot counter this contract! The status of contract is not Negotiable!")
             contract_to_counter.state = ContractModel.STATE_COUNTER
             signal_contract.send(sender=ContractModel, contract=contract_to_counter, user=self.user)
@@ -309,7 +317,7 @@ class Contract(object):
             # variable to check if we have right to amend contract
             state_right = self.__check_rights_by_status(contract_to_amend.state)
             # check if we can amend
-            if state_right is not "cannot_update" and contract_to_amend.state is not ContractModel.STATE_TERMINATED:
+            if state_right != "cannot_update" and contract_to_amend.state != ContractModel.STATE_TERMINATED:
                 raise ContractUpdateError("You cannot amend this contract!")
             # create copy of the contract
             amended_contract = copy(contract_to_amend)
@@ -370,7 +378,7 @@ class Contract(object):
             # block renewing contract not in Updateable or Approvable state
             state_right = self.__check_rights_by_status(contract_to_renew.state)
             # check if we can renew
-            if state_right is not "cannot_update" and contract_to_renew.state is not ContractModel.STATE_TERMINATED:
+            if state_right != "cannot_update" and contract_to_renew.state != ContractModel.STATE_TERMINATED:
                 raise ContractUpdateError("You cannot renew this contract!")
             # create copy of the contract - later we also copy contract detail
             renewed_contract = copy(contract_to_renew)
@@ -386,12 +394,12 @@ class Contract(object):
             renewed_contract.amount_rectified, renewed_contract.amount_due = (0, 0)
             renewed_contract.save(username=self.user.username)
             historical_record = renewed_contract.history.all().first()
-            renewed_contract.json_ext = json.dumps(_save_json_external(
-                user_id=historical_record.user_updated.id,
-                datetime=historical_record.date_updated,
+            renewed_contract.json_ext = _save_json_external(
+                user_id=str(historical_record.user_updated.id),
+                datetime=str(historical_record.date_updated),
                 message=f"contract renewed - state "
                         f"{historical_record.state}"
-            ), cls=DjangoJSONEncoder)
+            )
             renewed_contract.save(username=self.user.username)
             # copy also contract details
             self.__copy_details(contract_id=contract_id, modified_contract=renewed_contract)
@@ -436,12 +444,12 @@ class Contract(object):
                     contract.state = ContractModel.STATE_TERMINATED
                     contract.save(username=self.user.username)
                     historical_record = contract.history.all().first()
-                    contract.json_ext = json.dumps(_save_json_external(
-                        user_id=historical_record.user_updated.id,
-                        datetime=historical_record.date_updated,
+                    contract.json_ext = _save_json_external(
+                        user_id=str(historical_record.user_updated.id),
+                        datetime=str(historical_record.date_updated),
                         message=f"contract terminated - state "
                                 f"{historical_record.state}"
-                    ), cls=DjangoJSONEncoder)
+                    )
                     contract.save(username=self.user.username)
                 return {
                     "success": True,
@@ -464,9 +472,9 @@ class Contract(object):
                 raise PermissionError("Unauthorized")
 
             contract_output_list = []
-            payment_detail = PaymentDetail.objects.filter(
+            payment_detail = PaymentDetail.get_queryset(PaymentDetail.objects.filter(
                 payment__id=credit_note["id"]
-            ).prefetch_related(
+            )).prefetch_related(
                 'premium__contract_contribution_plan_details__contract_details__contract'
             ).prefetch_related(
                 'premium__contract_contribution_plan_details'
@@ -507,7 +515,7 @@ class ContractDetails(object):
             )
             for phi in policy_holder_insuree:
                 # TODO add the validity condition also!
-                if phi.is_deleted == False and phi.contribution_plan_bundle:
+                if phi.is_deleted is False and phi.contribution_plan_bundle:
                     cd = ContractDetailsModel(
                         **{
                             "contract_id": contract_details["contract_id"],
@@ -583,7 +591,7 @@ class ContractContributionPlanDetails(object):
             insuree=insuree,
             date_valid_from=ccpd.date_valid_from,
             date_valid_to=ccpd.date_valid_to,
-            product_id=ccpd.contribution_plan.benefit_plan.id,
+            product=ccpd.contribution_plan.benefit_plan,
         )
         return self.__create_contribution_from_policy(ccpd, policies)
 
@@ -605,52 +613,73 @@ class ContractContributionPlanDetails(object):
             ccpd.save(username=self.user.username)
             return [ccpd_new, ccpd]
 
-    def __get_policy(self, insuree, date_valid_from, date_valid_to, product_id):
+    def __get_policy(self, insuree, date_valid_from, date_valid_to, product):
         from core import datetime
         policy_output = []
         # get all policies related to the product and insuree
-        policies = Policy.objects.filter(product__id=product_id).filter(family__head_insuree=insuree)
-        # get covered policy
-        if len(list(policies)) > 0:
-            policies_covered = list(policies.filter(
-                Q(start_date__lte=date_valid_to, expiry_date__gte=date_valid_from)
-            ).order_by('start_date'))
+        policies = Policy.objects.filter(product=product) \
+            .filter(family__head_insuree=insuree) \
+            .filter(start_date__lte=date_valid_to, expiry_date__gte=date_valid_from)
+        # get covered policy, use count to run a COUNT query
+        if policies.count() > 0:
+            policies_covered = list(policies.order_by('start_date'))
         else:
             policies_covered = []
-
+        missing_coverage = []
+        # make sure the policies covers the contract : 
         last_date_covered = date_valid_from
+        # get the start date of the new contract by updating last_date_covered to the policy.stop_date
         while last_date_covered < date_valid_to and len(policies_covered) > 0:
             cur_policy = policies_covered.pop()
             # to check if it does take the first
-            if cur_policy.start_date < date_valid_from:
+            if cur_policy.start_date <= last_date_covered:
                 # Really unlikely we might create a policy that stop at curPolicy.startDate
                 # (start at curPolicy.startDate - product length) and add it to policy_output
                 last_date_covered = cur_policy.expiry_date
                 policy_output.append(cur_policy)
+            elif cur_policy.expiry_date <= date_valid_to:
+                missing_coverage.append({'start': cur_policy.start_date, 'stop': last_date_covered})
+                last_date_covered = cur_policy.expiry_date
+                policy_output.append(cur_policy)
+
+        for data in missing_coverage:
+            policy_created, last_date_covered = self.create_contract_details_policies(insuree, product, data['start'],
+                                                                                      data['stop'])
+            if policy_created is not None and len(policy_created) > 0:
+                policy_output += policy_created
 
         # now we create new policy
         while last_date_covered < date_valid_to:
-            # create policy for insuree familly
-            # TODO Policy with status - new open=32 in policy-be_py module
+            policy_created, last_date_covered = self.create_contract_details_policies(insuree, product,
+                                                                                      last_date_covered, date_valid_to)
+            if policy_created is not None and len(policy_created) > 0:
+                policy_output += policy_created
+        return policy_output
+
+    def create_contract_details_policies(self, insuree, product, last_date_covered, date_valid_to):
+        # create policy for insuree familly
+        # TODO Policy with status - new open=32 in policy-be_py module
+        policy_output = []
+        while last_date_covered < date_valid_to:
+            expiry_date = last_date_covered + relativedelta(months=product.insurance_period)
             cur_policy = Policy.objects.create(
                 **{
                     "family": insuree.family,
-                    "product_id": product_id,
+                    "product": product,
                     "status": Policy.STATUS_ACTIVE,
                     "stage": Policy.STAGE_NEW,
                     "enroll_date": last_date_covered,
                     "start_date": last_date_covered,
                     "validity_from": last_date_covered,
                     "effective_date": last_date_covered,
-                    "expiry_date": date_valid_to,
+                    "expiry_date": expiry_date,
                     "validity_to": None,
                     "audit_user_id": -1,
                 }
             )
-            last_date_covered = cur_policy.expiry_date
+            last_date_covered = expiry_date
             policy_output.append(cur_policy)
-
-        return policy_output
+        return policy_output, last_date_covered
 
     @check_authentication
     def contract_valuation(self, contract_contribution_plan_details):
@@ -707,7 +736,7 @@ class ContractContributionPlanDetails(object):
                 payment_id = payment_detail_contribution.payment.id
                 payment_object = Payment.objects.get(id=payment_id)
                 received_amount = payment_object.received_amount if payment_object.received_amount else 0
-                total_amount = total_amount - received_amount
+                total_amount = float(total_amount) - float(received_amount)
             dict_representation['total_amount'] = total_amount
             dict_representation['contribution_plan_details'] = ccpd_list
             return _output_result_success(dict_representation=dict_representation)
@@ -748,7 +777,7 @@ class ContractContributionPlanDetails(object):
             total_amount = total_amount - calculated_amount
             for ccpd_result in ccpd_results:
                 length_ccpd = float((ccpd_result.date_valid_to.year - ccpd_result.date_valid_from.year) * 12 \
-                              + (ccpd_result.date_valid_to.month - ccpd_result.date_valid_from.month))
+                                    + (ccpd_result.date_valid_to.month - ccpd_result.date_valid_from.month))
                 periodicity = float(ccpd_result.contribution_plan.periodicity)
                 # time part of splited as a fraction to count contribution value for that splited period properly
                 part_time_period = length_ccpd / periodicity
